﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.CombinedIntegerVectorEncoding {
  /// <summary>
  /// A base class for operators that perform a crossover of combined int-valued vectors.
  /// </summary>
  [Item("CombinedIntegerVectorCrossover", "A base class for operators that perform a crossover of combined int-valued vectors.")]
  [StorableClass]
  public abstract class CombinedIntegerVectorCrossover : SingleSuccessorOperator, IIntegerVectorOperator, ICombinedIntegerVectorCrossover, ICombinedIntegerVectorOperator, ICrossover, IStochasticOperator, IOperator, IParameterizedNamedItem, INamedItem, IParameterizedItem, IItem, IContent, IDeepCloneable, ICloneable {
    public override bool CanChangeName {
      get { return false; }
    }
    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<ItemArray<CombinedIntegerVector>> ParentsParameter {
      get { return (ScopeTreeLookupParameter<CombinedIntegerVector>)Parameters["Parents"]; }
    }
    public ILookupParameter<CombinedIntegerVector> ChildParameter {
      get { return (ILookupParameter<CombinedIntegerVector>)Parameters["Child"]; }
    }

    [StorableConstructor]
    protected CombinedIntegerVectorCrossover(bool deserializing) : base(deserializing) { }
    protected CombinedIntegerVectorCrossover(CombinedIntegerVectorCrossover original, Cloner cloner) : base(original, cloner) { }
    protected CombinedIntegerVectorCrossover()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used for stochastic crossover operators."));
      Parameters.Add(new ScopeTreeLookupParameter<CombinedIntegerVector>("Parents", "The parent vectors which should be crossed."));
      ParentsParameter.ActualName = "CombinedIntegerVector";
      Parameters.Add(new LookupParameter<CombinedIntegerVector>("Child", "The child vector resulting from the crossover."));
      ChildParameter.ActualName = "CombinedIntegerVector";
    }

    public sealed override IOperation Apply() {
      ChildParameter.ActualValue = Cross(RandomParameter.ActualValue, ParentsParameter.ActualValue);
      return base.Apply();
    }

    protected abstract CombinedIntegerVector Cross(IRandom random, ItemArray<CombinedIntegerVector> parents);
  }
}
